<?php

namespace Drupal\cl_server\Controller;

use Drupal\cl_components\Component\Component;
use Drupal\cl_components\Component\ComponentDiscovery;
use Drupal\cl_components\Exception\ComponentNotFoundException;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Provides an endpoint for Storybook to query.
 *
 * @see https://github.com/storybookjs/storybook/tree/next/app/server
 */
class ServerEndpointController extends ControllerBase {

  /**
   * Kill-switch to avoid caching the page.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */
  private KillSwitch $cacheKillSwitch;

  /**
   * The discovery service.
   *
   * @var \Drupal\cl_components\Component\ComponentDiscovery
   */
  private ComponentDiscovery $componentDiscovery;

  /**
   * Creates an object.
   *
   * @param \Drupal\Core\PageCache\ResponsePolicy\KillSwitch $cache_kill_switch
   *   The cache kill switch.
   * @param \Drupal\cl_components\Component\ComponentDiscovery $component_discovery
   *   The component discovery.
   */
  public function __construct(KillSwitch $cache_kill_switch, ComponentDiscovery $component_discovery) {
    $this->cacheKillSwitch = $cache_kill_switch;
    $this->componentDiscovery = $component_discovery;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    $cache_kill_switch = $container->get('page_cache_kill_switch');
    assert($cache_kill_switch instanceof KillSwitch);
    $component_discovery = $container->get(ComponentDiscovery::class);
    assert($component_discovery instanceof ComponentDiscovery);
    return new static($cache_kill_switch, $component_discovery);
  }

  /**
   * Render a Twig template from a Storybook component directory.
   */
  public function render(Request $request): array {
    $arguments = $this->getArguments($request);
    try {
      $component = $this->getComponent($request);
    }
    catch (ComponentNotFoundException $e) {
      throw new BadRequestHttpException('Invalid component', $e);
    }
    $variant = $request->query->get('_variant');
    // Storybook will not allow empty title for the story. Let's consider
    // Default to be the default.
    $variant = $variant === 'default' ? '' : $variant;
    $build = [
      '#type' => 'cl_component',
      '#component' => $component->getMetadata()->getMachineName(),
      '#variant' => $variant,
      '#context' => $arguments,
    ];
    $this->cacheKillSwitch->trigger();
    return [
      '#attached' => [
        'library' => ['cl_components/attach_behaviors'],
      ],
      '#type' => 'container',
      '#cache' => ['max-age' => 0],
      // Magic wrapper ID to pull the HTML from.
      '#attributes' => ['id' => '___cl-wrapper'],
      'component' => $build,
    ];
  }

  /**
   * Gets the arguments.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The inbound request.
   *
   * @return array
   *   The array of arguments.
   */
  private function getArguments(Request $request): array {
    return Json::decode($request->query->get('_params'));
  }

  /**
   * Get the component based on the request object.
   *
   * @throws \Drupal\cl_components\Exception\ComponentNotFoundException
   *   If the component cannot be found.
   */
  public function getComponent(Request $request): Component {
    $story_filename = $request->query->get('_storyFileName');
    if (!$story_filename) {
      throw new ComponentNotFoundException('Impossible to find a story with an empty story file name.');
    }
    return $this->componentDiscovery->findBySiblingFile($story_filename);
  }

}
